##==============================================================================
##~For creating a shell script in either an existing project or in ShSDK.
##~
##~usage: shcf_cli bin <project_name> <script_name>
##~  project_name    * existing project must be listed in 'shcf_cli projects'
##~                  * 'shcf' or '-' would mean relative to ShSDK root
##~
##==============================================================================

create_bin() {
    local project=$1
    local new_script=$2

    autoload_functions "usage define_rootpath"
   
    #show help if no parameters given
    [[ $# -lt 2 ]] && { echo "ERROR: Parameter(s) missing!"; usage bin; exit $FAILED; }
 
    local shortname_newscript=$($BASENAME_BIN ${new_script})
    local script_fullrootdir=$(define_rootpath $project)'/bin'
    local script_fullpath=$script_fullrootdir/$shortname_newscript

    ## make sure not to overwrite existing bin scripts
    [[ -e $script_fullpath ]] && {
       echo "ERROR: $script_fullpath exists. Aborting."
       return
    }

$CAT_BIN > $script_fullpath << EOL
### -------------------------------------------------
### ${shortname_newscript}: auto-generated by shcf_cli
### INPUT:
### OUTPUT:
### AUTHOR:
### GEN TIMESTAMP: `date`
### -------------------------------------------------
#!/bin/bash

## -------- source common library ---------
PROJECT_ROOTDIR="\$( cd "\$( dirname "\${BASH_SOURCE[0]}" )"/../ && pwd )"
. \$PROJECT_ROOTDIR/etc/controller.bash.inc


## list of functions to auto-load
## of course, edit function names
##
function_to_autoload="
rename_function1
rename_function2
rename_functionX
"

autoload_functions "\$function_to_autoload"


## ----- script parameter
##
param1=\$1
param2=\$2

## ..............................
## main utility tool starts below
## ..............................

EOL

if [[ -e $script_fullpath ]]; then
    echo "template script created in $script_fullpath"
else
    echo "creation error for $script_fullpath"
fi

} ## END: create_bin()
